@@ -35,11 +35,14 @@
 #define CONFIG_KEY_VIRTADDR		"virtual-addr"
 #define CONFIG_KEY_VIRTPORTS		"virtual-ports"
 #define CONFIG_KEY_IPADDR		"ip-addr"
-#define CONFIG_KEY_SRCADDR		"src-addr"
-#define CONFIG_KEY_PORTS		"ports"
+#define CONFIG_KEY_SRCADDR		"source-addr"
+#define CONFIG_KEY_PORT			"port"
 #define CONFIG_KEY_MODE			"mode"
 #define CONFIG_KEY_PROTO		"protocol"
 #define CONFIG_KEY_SCHED		"scheduler"
+#define CONFIG_KEY_SCHEDPARAM	"sched-param"
+#define CONFIG_KEY_PERSIST		"persistence"
+#define CONFIG_KEY_PERSISTTM	"persist-ttl"
 #define CONFIG_KEY_HELPER		"helper"
 #define CONFIG_KEY_LOG			"log"
 #define CONFIG_KEY_MARK			"mark"
@@ -48,6 +51,19 @@
 #define CONFIG_KEY_WEIGHT		"weight"
 #define CONFIG_KEY_PRIORITY		"priority"
 #define CONFIG_KEY_ACTION		"action"
+#define CONFIG_KEY_NEWRTLIMIT	"new-rtlimit"
+#define CONFIG_KEY_NEWRTLIMITBURST	"new-rtlimit-burst"
+#define CONFIG_KEY_RSTRTLIMIT	"rst-rtlimit"
+#define CONFIG_KEY_RSTRTLIMITBURST	"rst-rtlimit-burst"
+#define CONFIG_KEY_ESTCONNLIMIT	"est-connlimit"
+#define CONFIG_KEY_TCPSTRICT	"tcp-strict"
+#define CONFIG_KEY_QUEUE		"queue"
+#define CONFIG_KEY_POLICIES		"policies"
+#define CONFIG_KEY_TYPE			"type"
+#define CONFIG_KEY_TIMEOUT		"timeout"
+#define CONFIG_KEY_ELEMENTS		"elements"
+#define CONFIG_KEY_DATA			"data"
+#define CONFIG_KEY_TIME			"time"
 
 #define CONFIG_VALUE_FAMILY_IPV4	"ipv4"
 #define CONFIG_VALUE_FAMILY_IPV6	"ipv6"
@@ -64,6 +80,13 @@
 #define CONFIG_VALUE_SCHED_WEIGHT	"weight"
 #define CONFIG_VALUE_SCHED_HASH		"hash"
 #define CONFIG_VALUE_SCHED_SYMHASH	"symhash"
+#define CONFIG_VALUE_META_NONE		"none"
+#define CONFIG_VALUE_META_SRCIP		"srcip"
+#define CONFIG_VALUE_META_DSTIP		"dstip"
+#define CONFIG_VALUE_META_SRCPORT	"srcport"
+#define CONFIG_VALUE_META_DSTPORT 	"dstport"
+#define CONFIG_VALUE_META_SRCMAC	"srcmac"
+#define CONFIG_VALUE_META_DSTMAC	"dstmac"
 #define CONFIG_VALUE_HELPER_NONE		"none"
 #define CONFIG_VALUE_HELPER_AMANDA		"amanda"
 #define CONFIG_VALUE_HELPER_FTP			"ftp"
@@ -88,6 +111,19 @@
 #define CONFIG_VALUE_ACTION_START	"start"
 #define CONFIG_VALUE_ACTION_RELOAD	"reload"
 #define CONFIG_VALUE_ACTION_NONE	"none"
+#define CONFIG_VALUE_SWITCH_ON		"on"
+#define CONFIG_VALUE_SWITCH_OFF		"off"
+#define CONFIG_VALUE_POLICIES_TYPE_BL	"blacklist"
+#define CONFIG_VALUE_POLICIES_TYPE_WL	"whitelist"
+
+enum config_parser_output {
+	PARSER_OK,
+	PARSER_FAILED,
+	PARSER_STRUCT_FAILED,
+	PARSER_VALID_FAILED,
+	PARSER_IDEM_VALUE,
+	PARSER_OBJ_UNKNOWN,
+};
 
 enum config_src {
 	CONFIG_SRC_FILE,
@@ -101,11 +137,16 @@ struct config_pair {
 	int		int_value;
 };
 
+void config_pair_init(struct config_pair *c);
 int config_file(const char *file);
 int config_buffer(const char *buf);
 int config_print_farms(char **buf, char *name);
+int config_print_policies(char **buf, char *name);
 int config_set_farm_action(const char *name, const char *value);
 int config_set_backend_action(const char *fname, const char *bname, const char *value);
+int config_set_fpolicy_action(const char *fname, const char *fpname, const char *value);
+int config_set_policy_action(const char *name, const char *value);
+int config_set_element_action(const char *pname, const char *edata, const char *value);
 void config_print_response(char **buf, const char *message);
 
 #endif /* _CONFIG_H_ */
