@@ -26,6 +26,8 @@
 
 #define NFTLB_POSTROUTING_MARK		0x80000000
 
+int nft_reset(void);
 int nft_rulerize(struct farm *f);
+int nft_rulerize_policies(struct policy *p);
 
 #endif /* _NFT_H_ */
