@@ -69,7 +69,7 @@ static const struct option options[] = {
 
 static void nftlb_sighandler(int signo)
 {
-	syslog(LOG_INFO, "shuting down %s, bye", PACKAGE);
+	syslog(LOG_INFO, "shutting down %s, bye", PACKAGE);
 	server_fini();
 	exit(EXIT_SUCCESS);
 }
@@ -94,6 +94,7 @@ int main(int argc, char *argv[])
 			break;
 		case 'k':
 			server_set_key(optarg);
+			sprintf(optarg, "%*s", (int)strlen(optarg) - 1, " ");
 			break;
 		case 'e':
 			mode = NFTLB_EXIT_MODE;
@@ -119,6 +120,8 @@ int main(int argc, char *argv[])
 	    signal(SIGPIPE, SIG_IGN) == SIG_ERR)
 		return EXIT_FAILURE;
 
+	setlogmask(LOG_UPTO(loglevel));
+
 	objects_init();
 
 	loop_init();
