@@ -32,6 +32,7 @@
 #include "config.h"
 #include "nft.h"
 #include "events.h"
+#include "sbuffer.h"
 
 #define SRV_MAX_BUF			40960
 #define SRV_MAX_HEADER			300
@@ -61,7 +62,9 @@ enum ws_methods {
 
 enum ws_responses {
 	WS_HTTP_500,
+	WS_HTTP_400,
 	WS_HTTP_401,
+	WS_HTTP_404,
 	WS_HTTP_200,
 };
 
@@ -75,7 +78,9 @@ struct nftlb_http_state {
 
 static const char *ws_str_responses[] = {
 	HTTP_PROTO "500 Internal Server Error" HTTP_LINE_END,
+	HTTP_PROTO "400 Bad Request" HTTP_LINE_END,
 	HTTP_PROTO "401 Unauthorized" HTTP_LINE_END,
+	HTTP_PROTO "404 Not Found" HTTP_LINE_END,
 	HTTP_PROTO "200 OK" HTTP_LINE_END,
 };
 
@@ -99,13 +104,20 @@ static int auth_key(const char *recvkey)
 	return (strcmp(nftserver.key, recvkey) == 0);
 }
 
-static int get_request(char *buf, struct nftlb_http_state *state)
+static int get_request(int fd, struct sbuffer *buf, struct nftlb_http_state *state)
 {
 	char method[SRV_MAX_IDENT] = {0};
 	char strkey[SRV_MAX_IDENT] = {0};
+	int contlength = 0;
+	int times = 0;
+	int total_read_size = 0;
 	char *ptr;
+	int size;
+	int head;
+	int bytes_left;
+	int cont_100 = 0;
 
-	if ((ptr = strstr(buf, "Key: ")) == NULL) {
+	if ((ptr = strstr(get_buf_data(buf), "Key: ")) == NULL) {
 		state->status_code = WS_HTTP_401;
 		return -1;
 	}
@@ -117,7 +129,7 @@ static int get_request(char *buf, struct nftlb_http_state *state)
 		return -1;
 	}
 
-	sscanf(buf, "%199[^ ] %199[^ ] ", method, state->uri);
+	sscanf(get_buf_data(buf), "%199[^ ] %199[^ ] ", method, state->uri);
 
 	if (strncmp(method, STR_GET_ACTION, 3) == 0) {
 		state->method = WS_GET_ACTION;
@@ -136,28 +148,78 @@ static int get_request(char *buf, struct nftlb_http_state *state)
 	    state->method != WS_PUT_ACTION)
 		return 0;
 
-	state->body = strstr(buf, "\r\n\r\n");
-	if (state->body)
-		state->body += 4;
+	state->body = strstr(get_buf_data(buf), "\r\n\r\n");
+	if (!state->body) {
+		syslog(LOG_ERR, "Not found body section in the request");
+		state->status_code = WS_HTTP_400;
+		return -1;
+	}
+	state->body += 4;
+	head = state->body - get_buf_data(buf);
+
+	if ((ptr = strstr(get_buf_data(buf), "Expect: 100-continue")) != NULL) {
+		cont_100 = 1;
+		send(fd, "HTTP/1.1 100 Continue\r\n\r\n", 25, 0);
+	}
+
+	if ((ptr = strstr(get_buf_data(buf), "Content-Length: ")) != NULL) {
+		sscanf(ptr, "Content-Length: %i[^\r\n]", &contlength);
+
+		if (head + contlength >= get_buf_size(buf))
+			times = ((head + contlength - get_buf_size(buf)) / EXTRA_SIZE) + 1;
+		if (times == 0)
+			goto receive;
+
+		if (resize_buf(buf, times)) {
+			syslog(LOG_ERR, "Error resizing the buffer %d times from a size of %d!", times, get_buf_size(buf));
+			state->status_code = WS_HTTP_500;
+			return -1;
+		}
+	}
+
+receive:
+	state->body = get_buf_data(buf) + head;
+	total_read_size = get_buf_next(buf) - state->body;
+	while ((total_read_size < contlength) || cont_100) {
+		cont_100 = 0;
+		bytes_left = EXTRA_SIZE;
+		if (contlength - total_read_size < EXTRA_SIZE)
+			bytes_left = contlength - total_read_size;
+		if (bytes_left <= 0)
+			goto final;
+		size = recv(fd, get_buf_next(buf), bytes_left, 0);
+		if (size <= 0)
+			goto final;
+		buf->next += size;
+		total_read_size += size;
+	}
+
+final:
+	concat_buf(buf, "\0");
 
 	return 0;
 }
 
 static int send_get_response(struct nftlb_http_state *state)
 {
-	char farm[SRV_MAX_IDENT] = {0};
-	char farms[SRV_MAX_IDENT] = {0};
+	char firstlevel[SRV_MAX_IDENT] = {0};
+	char secondlevel[SRV_MAX_IDENT] = {0};
+	char thirdlevel[SRV_MAX_IDENT] = {0};
+	char fourthlevel[SRV_MAX_IDENT] = {0};
 
-	sscanf(state->uri, "/%199[^/]/%199[^/\n]", farms, farm);
+	sscanf(state->uri, "/%199[^/]/%199[^/]/%199[^/]/%199[^\n]",
+	       firstlevel, secondlevel, thirdlevel, fourthlevel);
 
-	if (strcmp(farms, CONFIG_KEY_FARMS) != 0) {
-		state->status_code = WS_HTTP_500;
-		return -1;
-	}
-
-	if (config_print_farms(&state->body_response, farm) == 0) {
-		state->status_code = WS_HTTP_200;
-		return 0;
+	if (strcmp(firstlevel, CONFIG_KEY_FARMS) == 0) {
+		if (config_print_farms(&state->body_response, secondlevel) == 0) {
+			state->status_code = WS_HTTP_200;
+			return 0;
+		}
+	} else if (strcmp(firstlevel, CONFIG_KEY_POLICIES) == 0) {
+		if (config_print_policies(&state->body_response, secondlevel) == 0) {
+			state->status_code = WS_HTTP_200;
+			return 0;
+		}
 	}
 
 	state->status_code = WS_HTTP_500;
@@ -166,64 +228,97 @@ static int send_get_response(struct nftlb_http_state *state)
 
 static int send_delete_response(struct nftlb_http_state *state)
 {
-	char farm[SRV_MAX_IDENT] = {0};
-	char bck[SRV_MAX_IDENT] = {0};
-	char farms[SRV_MAX_IDENT] = {0};
-	char bcks[SRV_MAX_IDENT] = {0};
+	char firstlevel[SRV_MAX_IDENT] = {0};
+	char secondlevel[SRV_MAX_IDENT] = {0};
+	char thirdlevel[SRV_MAX_IDENT] = {0};
+	char fourthlevel[SRV_MAX_IDENT] = {0};
 	int ret;
 
 	sscanf(state->uri, "/%199[^/]/%199[^/]/%199[^/]/%199[^\n]",
-	       farms, farm, bcks, bck);
+	       firstlevel, secondlevel, thirdlevel, fourthlevel);
 
-	if (strcmp(farms, CONFIG_KEY_FARMS) != 0) {
+	if (strcmp(firstlevel, CONFIG_KEY_FARMS) != 0 &&
+		strcmp(firstlevel, CONFIG_KEY_POLICIES) != 0) {
 		state->status_code = WS_HTTP_500;
 		return -1;
 	}
 
-	state->body_response = malloc(SRV_MAX_IDENT);
+	state->body_response = malloc(SRV_MAX_BUF);
 	if (!state->body_response) {
 		state->status_code = WS_HTTP_500;
 		return -1;
 	}
 
-	if (strcmp(bcks,CONFIG_KEY_BCKS) == 0) {
-		ret = config_set_backend_action(farm, bck, CONFIG_VALUE_ACTION_DELETE);
-		if (ret != 0) {
+	if (strcmp(firstlevel, CONFIG_KEY_FARMS) == 0 &&
+		strcmp(thirdlevel, CONFIG_KEY_BCKS) == 0) {
+		ret = config_set_backend_action(secondlevel, fourthlevel, CONFIG_VALUE_ACTION_STOP);
+		if (ret > 0)
+			ret = config_set_farm_action(secondlevel, CONFIG_VALUE_ACTION_RELOAD);
+		if (ret > 0)
+			obj_rulerize();
+		ret = config_set_backend_action(secondlevel, fourthlevel, CONFIG_VALUE_ACTION_DELETE);
+		if (ret < 0) {
 			config_print_response(&state->body_response,
 					      "error deleting backend");
 			goto delete_end;
 		}
-		ret = config_set_farm_action(farm, CONFIG_VALUE_ACTION_RELOAD);
-		if (ret != 0) {
+	} else if (strcmp(firstlevel, CONFIG_KEY_FARMS) == 0 &&
+			   strcmp(thirdlevel, CONFIG_KEY_POLICIES) == 0) {
+		ret = config_set_farm_action(secondlevel, CONFIG_VALUE_ACTION_RELOAD);
+		if (ret < 0) {
 			config_print_response(&state->body_response,
 					      "error reloading farm");
 			goto delete_end;
 		}
-		ret = farm_s_rulerize();
+		ret = config_set_fpolicy_action(secondlevel, fourthlevel, CONFIG_VALUE_ACTION_DELETE);
 		if (ret != 0) {
 			config_print_response(&state->body_response,
-					      "error generating rules");
+					      "error stopping farm policy");
 			goto delete_end;
 		}
-	} else {
-		ret = config_set_farm_action(farm, CONFIG_VALUE_ACTION_STOP);
-		if (ret != 0) {
+		obj_rulerize();
+	} else if (strcmp(firstlevel, CONFIG_KEY_POLICIES) == 0 &&
+			   strcmp(thirdlevel, CONFIG_KEY_ELEMENTS) == 0) {
+		ret = config_set_element_action(secondlevel, fourthlevel, CONFIG_VALUE_ACTION_STOP);
+		if (ret > 0)
+			ret = config_set_policy_action(secondlevel, CONFIG_VALUE_ACTION_RELOAD);
+		if (ret > 0)
+			obj_rulerize();
+		ret = config_set_element_action(secondlevel, fourthlevel, CONFIG_VALUE_ACTION_DELETE);
+		if (ret < 0) {
 			config_print_response(&state->body_response,
-					      "error stopping farm");
+					      "error deleting policy element");
 			goto delete_end;
 		}
-		ret = farm_s_rulerize();
-		if (ret != 0) {
-			config_print_response(&state->body_response,
-					      "error generating rules");
-			goto delete_end;
-		}
-		config_set_farm_action(farm, CONFIG_VALUE_ACTION_DELETE);
-		if (ret != 0) {
+	} else if (strcmp(firstlevel, CONFIG_KEY_FARMS) == 0 &&
+			   strcmp(thirdlevel, "") == 0) {
+		ret = config_set_farm_action(secondlevel, CONFIG_VALUE_ACTION_STOP);
+		if (ret > 0)
+			obj_rulerize();
+		ret = config_set_farm_action(secondlevel, CONFIG_VALUE_ACTION_DELETE);
+		if (ret < 0) {
 			config_print_response(&state->body_response,
 					      "error deleting farm");
 			goto delete_end;
 		}
+	} else if (strcmp(firstlevel, CONFIG_KEY_POLICIES) == 0 &&
+			   strcmp(thirdlevel, "") == 0) {
+		ret = config_set_policy_action(secondlevel, CONFIG_VALUE_ACTION_STOP);
+		if (ret < 0) {
+			config_print_response(&state->body_response,
+					      "error stopping policy");
+			goto delete_end;
+		}
+		obj_rulerize();
+		ret = config_set_policy_action(secondlevel, CONFIG_VALUE_ACTION_DELETE);
+		if (ret < 0) {
+			config_print_response(&state->body_response,
+					      "error deleting policy");
+			goto delete_end;
+		}
+	} else {
+		state->status_code = WS_HTTP_500;
+		return -1;
 	}
 
 	config_print_response(&state->body_response, "success");
@@ -235,30 +330,49 @@ delete_end:
 
 static int send_post_response(struct nftlb_http_state *state)
 {
-	if (strncmp(state->uri, "/farms", 6) != 0) {
-		state->status_code = WS_HTTP_500;
+	char firstlevel[SRV_MAX_IDENT] = {0};
+
+	sscanf(state->uri, "/%199[^\n]", firstlevel);
+
+	if ((strcmp(firstlevel, CONFIG_KEY_FARMS) != 0) &&
+		(strcmp(firstlevel, CONFIG_KEY_POLICIES) != 0)) {
+		state->status_code = WS_HTTP_404;
 		return -1;
 	}
 
-	state->body_response = malloc(SRV_MAX_IDENT);
+	state->body_response = malloc(SRV_MAX_BUF);
 	if (!state->body_response) {
 		state->status_code = WS_HTTP_500;
 		return -1;
 	}
 
-	if (config_buffer(state->body_response) != 0) {
+	switch (config_buffer(state->body)) {
+	case PARSER_OK:
+		break;
+	case PARSER_STRUCT_FAILED:
+		config_print_response(&state->body_response,
+				      "the structure is invalid");
+		goto post_end;
+		break;
+	case PARSER_OBJ_UNKNOWN:
+		config_print_response(&state->body_response,
+				      "the object to modify is unknown");
+		goto post_end;
+		break;
+	default:
 		config_print_response(&state->body_response,
 				      "error parsing buffer");
 		goto post_end;
+		break;
 	}
 
-	if (farm_s_rulerize() != 0) {
+	if (obj_rulerize() != 0) {
 		config_print_response(&state->body_response,
 				      "error generating rules");
 		goto post_end;
 	}
 
-	config_print_response(&state->body, "success");
+	config_print_response(&state->body_response, "success");
 
 post_end:
 	state->status_code = WS_HTTP_200;
@@ -309,7 +423,7 @@ static void nftlb_http_send_response(struct ev_io *io,
 
 static void nftlb_read_cb(struct ev_loop *loop, struct ev_io *io, int revents)
 {
-	char buffer[SRV_MAX_BUF] = {0};
+	struct sbuffer buf;
 	struct nftlb_http_state state;
 	struct nftlb_client *cli;
 	ssize_t size;
@@ -320,16 +434,21 @@ static void nftlb_read_cb(struct ev_loop *loop, struct ev_io *io, int revents)
 	}
 	cli = container_of(io, struct nftlb_client, io);
 
-	size = recv(io->fd, buffer, SRV_MAX_BUF, 0);
+	state.body_response = NULL;
+	create_buf(&buf);
+	size = recv(io->fd, get_buf_data(&buf), DEFAULT_BUFFER_SIZE - 1, 0);
 	if (size < 0)
 		return;
+
+	buf.next = size;
+
 	if (size == 0) {
 		syslog(LOG_DEBUG, "connection closed by client %s:%hu\n",
 		       inet_ntoa(cli->addr.sin_addr), ntohs(cli->addr.sin_port));
 		goto end;
 	}
 
-	if (get_request(buffer, &state) < 0) {
+	if (get_request(io->fd, &buf, &state) < 0) {
 		nftlb_http_send_response(io, &state, 0);
 		goto end;
 	}
@@ -345,13 +464,15 @@ static void nftlb_read_cb(struct ev_loop *loop, struct ev_io *io, int revents)
 	syslog(LOG_DEBUG, "connection closed by server %s:%hu\n",
 	       inet_ntoa(cli->addr.sin_addr), ntohs(cli->addr.sin_port));
 end:
-	bzero(buffer, size);
-	free(state.body_response);
+	if (state.body_response)
+		free(state.body_response);
+	clean_buf(&buf);
 
 	ev_timer_stop(loop, &cli->timer);
 	nftlb_client_release(loop, cli);
 
 	syslog(LOG_DEBUG, "%d client(s) connected", --nftserver.clients);
+
 	return;
 }
 
@@ -457,7 +578,7 @@ void server_fini(void)
 
 void server_set_host(char *host)
 {
-	nftserver.host = malloc(strlen(host));
+	nftserver.host = malloc(strlen(host)+1);
 
 	if (!nftserver.host) {
 		syslog(LOG_ERR, "No memory available to allocate the server host");
